# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Test the darwin utils functions."""

import os
import sys

from twisted.internet import defer

from ubuntuone.controlpanel import utils
from ubuntuone.devtools.testcases import TestCase

# let me use protected methods
# pylint: disable=W0212


class InstallConfigTestCase(TestCase):
    """Test install_config_and_daemons."""

    @defer.inlineCallbacks
    def setUp(self):
        """Set up multi-call checker."""
        yield super(InstallConfigTestCase, self).setUp()
        self._called = []

    def _set_called(self, *args, **kwargs):
        """Store 'args' and 'kwargs for test assertions."""
        self._called.append((args, kwargs))

    def test_do_nothing_unfrozen(self):
        """Test that install_config_and_daemons does nothing when unfrozen."""

        self.patch(utils.darwin, 'save_config_path',
                   self._set_called)
        utils.install_config_and_daemons()
        self.assertEqual(self._called, [])

    def _test_copying_conf_files(self, exists):
        """Call install_config_and_daemons, parameterize os.path.exists."""
        sys.frozen = 'macosx_app'
        self.addCleanup(delattr, sys, 'frozen')
        self.patch(utils.darwin, 'save_config_path', lambda x: "TARGET_PATH")
        self.patch(utils.darwin, '__file__', "/path/to/Main.app/ignore")
        self.patch(os.path, "exists", lambda x: exists)
        self.patch(utils.darwin.shutil, 'copyfile',
                   self._set_called)
        utils.install_config_and_daemons()

    def test_copies_conf_files(self):
        """When frozen, we copy the conf files if they don't exist."""
        self._test_copying_conf_files(False)
        self.assertEqual(self._called,
                         [(('/path/to/Main.app/Contents/'
                           'Resources/syncdaemon.conf',
                           'TARGET_PATH/syncdaemon.conf'), {}),
                          (('/path/to/Main.app/Contents/'
                            'Resources/logging.conf',
                            'TARGET_PATH/logging.conf'),
                           {})])

    def test_does_not_copy_conf_files(self):
        """When frozen, we do not copy the conf files if they do exist."""
        self._test_copying_conf_files(True)
        self.assertEqual(self._called, [])
